@@ -398,7 +398,8 @@ class StructuralPlacementListener(walker.RDLListener):
 
 
     def exit_Addrmap(self, node: AddrmapNode) -> None:
-        self.resolve_addresses(node)
+        is_bridge = node.get_property('bridge')
+        self.resolve_addresses(node, is_bridge)
 
         self.msb0_mode_stack.pop()
         self.addressing_mode_stack.pop()
@@ -419,7 +420,7 @@ class StructuralPlacementListener(walker.RDLListener):
                 )
 
 
-    def resolve_addresses(self, node: AddressableNode) -> None:
+    def resolve_addresses(self, node: AddressableNode, is_bridge: bool = False) -> None:
         """
         Resolve addresses of children of Addrmap and Regfile components
         """
@@ -483,7 +484,7 @@ class StructuralPlacementListener(walker.RDLListener):
 
             # Calculate resulting address offset
             alignment = max(prop_alignment, alloc_alignment, mode_alignment)
-            if prev_node is None:
+            if (prev_node is None) or is_bridge:
                 next_offset = 0
             else:
                 next_offset = prev_node.raw_address_offset + prev_node.total_size
@@ -556,11 +557,11 @@ class LateElabListener(walker.RDLListener):
 
             # Augment based on paramter overrides as per 5.1.1.4
             if node.inst.original_def is not None:
-                for i in range(len(node.inst.parameters)):
+                for i, inst_parameter in enumerate(node.inst.parameters):
                     orig_param_value = node.inst.original_def.parameters[i].get_value()
-                    new_param_value = node.inst.parameters[i].get_value()
+                    new_param_value = inst_parameter.get_value()
                     if new_param_value != orig_param_value:
-                        extra_type_name_segments.append(node.inst.parameters[i].get_normalized_parameter())
+                        extra_type_name_segments.append(inst_parameter.get_normalized_parameter())
 
             # Further augment type name as per extended type generation from DPAs
             if self.env.use_extended_type_name_gen:
