@@ -8,8 +8,11 @@ from . import expressions
 if TYPE_CHECKING:
     from ..compiler import RDLEnvironment
     from ..source_ref import SourceRefBase
+    from typing import TypeVar
 
-def get_all_subclasses(cls: type) -> List[type]:
+    T = TypeVar('T')
+
+def get_all_subclasses(cls: Type['T']) -> List[Type['T']]:
     return cls.__subclasses__() + [
         g for s in cls.__subclasses__()
         for g in get_all_subclasses(s)
@@ -39,8 +42,12 @@ class PropertyRule:
         self.env = env
 
     #---------------------------------------------------------------------------
+    @classmethod
+    def get_name_cls(cls) -> str:
+        return cls.__name__.replace("Prop_", "")
+
     def get_name(self) -> str:
-        return type(self).__name__.replace("Prop_", "")
+        return self.get_name_cls()
 
     #---------------------------------------------------------------------------
     def assign_value(self, comp_def: comp.Component, value: Any, src_ref: 'SourceRefBase') -> None:
@@ -100,6 +107,13 @@ class PropertyRule:
                     src_ref
                 )
 
+        # If the property belongs to a mutex group, wipe out any of its
+        # counterpart properties
+        if self.mutex_group is not None:
+            for prop_name in MUTEX_PROP_GROUPS[self.mutex_group]:
+                if prop_name in comp_def.properties:
+                    del comp_def.properties[prop_name]
+
         # Store the property
         comp_def.properties[self.get_name()] = value
 
@@ -166,16 +180,6 @@ class PropertyRuleBoolPair(PropertyRule):
     # Property name of the equivalent opposite
     opposite_property = ""
 
-
-    def assign_value(self, comp_def: comp.Component, value: Any, src_ref: 'SourceRefBase') -> None:
-        """
-        Side effect: Ensure assignment of the opposite is cleared since it is being
-        overridden
-        """
-        super().assign_value(comp_def, value, src_ref)
-        if self.opposite_property in comp_def.properties:
-            del comp_def.properties[self.opposite_property]
-
     def get_default(self, node: m_node.Node) -> bool:
         """
         If not explicitly set, check if the opposite was set first before returning
@@ -614,6 +618,20 @@ class Prop_resetsignal(PropertyRule):
     def validate(self, node: m_node.Node, value: Any) -> None:
         self._validate_ref_width_is_1(node, "resetsignal", value)
 
+    def get_default(self, node: m_node.Node) -> Optional[m_node.SignalNode]:
+        """
+        If no field reset signal was explicitly assigned, search for signals in
+        the enclosing hierarchy with field_reset=True
+        """
+        current_node = node
+        while current_node is not None:
+            for signal in current_node.signals():
+                if signal.get_property('field_reset'):
+                    return signal
+            current_node = current_node.parent
+
+        return None
+
 #-------------------------------------------------------------------------------
 # Software access properties
 #-------------------------------------------------------------------------------
@@ -629,16 +647,6 @@ class Prop_rclr(PropertyRule):
     dyn_assign_allowed = True
     mutex_group = "P"
 
-    def assign_value(self, comp_def: comp.Component, value: Any, src_ref: 'SourceRefBase') -> None:
-        """
-        Overrides other related properties
-        """
-        super().assign_value(comp_def, value, src_ref)
-        if "rset" in comp_def.properties:
-            del comp_def.properties["rset"]
-        if "onread" in comp_def.properties:
-            del comp_def.properties["onread"]
-
     def get_default(self, node: m_node.Node) -> bool:
         """
         If not explicitly set, check if onread sets the equivalent
@@ -659,16 +667,6 @@ class Prop_rset(PropertyRule):
     dyn_assign_allowed = True
     mutex_group = "P"
 
-    def assign_value(self, comp_def: comp.Component, value: Any, src_ref: 'SourceRefBase') -> None:
-        """
-        Overrides other related properties
-        """
-        super().assign_value(comp_def, value, src_ref)
-        if "rclr" in comp_def.properties:
-            del comp_def.properties["rclr"]
-        if "onread" in comp_def.properties:
-            del comp_def.properties["onread"]
-
     def get_default(self, node: m_node.Node) -> bool:
         """
         If not explicitly set, check if onread sets the equivalent
@@ -689,16 +687,6 @@ class Prop_onread(PropertyRule):
     dyn_assign_allowed = True
     mutex_group = "P"
 
-    def assign_value(self, comp_def: comp.Component, value: Any, src_ref: 'SourceRefBase') -> None:
-        """
-        Overrides other related properties
-        """
-        super().assign_value(comp_def, value, src_ref)
-        if "rclr" in comp_def.properties:
-            del comp_def.properties["rclr"]
-        if "rset" in comp_def.properties:
-            del comp_def.properties["rset"]
-
     def get_default(self, node: m_node.Node) -> Optional[rdltypes.OnReadType]:
         """
         If not explicitly set, check if rset or rclr imply the value
@@ -740,16 +728,6 @@ class Prop_woclr(PropertyRule):
     dyn_assign_allowed = True
     mutex_group = "B"
 
-    def assign_value(self, comp_def: comp.Component, value: Any, src_ref: 'SourceRefBase') -> None:
-        """
-        Overrides other related properties
-        """
-        super().assign_value(comp_def, value, src_ref)
-        if "woset" in comp_def.properties:
-            del comp_def.properties["woset"]
-        if "onwrite" in comp_def.properties:
-            del comp_def.properties["onwrite"]
-
     def get_default(self, node: m_node.Node) -> bool:
         """
         If not explicitly set, check if onwrite sets the equivalent
@@ -770,16 +748,6 @@ class Prop_woset(PropertyRule):
     dyn_assign_allowed = True
     mutex_group = "B"
 
-    def assign_value(self, comp_def: comp.Component, value: Any, src_ref: 'SourceRefBase') -> None:
-        """
-        Overrides other related properties
-        """
-        super().assign_value(comp_def, value, src_ref)
-        if "woclr" in comp_def.properties:
-            del comp_def.properties["woclr"]
-        if "onwrite" in comp_def.properties:
-            del comp_def.properties["onwrite"]
-
     def get_default(self, node: m_node.Node) -> bool:
         """
         If not explicitly set, check if onread sets the equivalent
@@ -800,16 +768,6 @@ class Prop_onwrite(PropertyRule):
     dyn_assign_allowed = True
     mutex_group = "B"
 
-    def assign_value(self, comp_def: comp.Component, value: Any, src_ref: 'SourceRefBase') -> None:
-        """
-        Overrides other related properties
-        """
-        super().assign_value(comp_def, value, src_ref)
-        if "woclr" in comp_def.properties:
-            del comp_def.properties["woclr"]
-        if "woset" in comp_def.properties:
-            del comp_def.properties["woset"]
-
     def get_default(self, node: m_node.Node) -> Optional[rdltypes.OnWriteType]:
         """
         If not explicitly set, check if woset or woclr imply the value
@@ -1315,7 +1273,8 @@ class Prop_intr_type(PropertyRule):
     dyn_assign_allowed = True
     mutex_group = None
 
-    def get_name(self) -> str:
+    @classmethod
+    def get_name_cls(cls) -> str:
         # Interrupt modifier type is a "special" hidden property
         # Intentinally override the property name to something that is impossible
         # to define in RDL and collide with
@@ -1460,6 +1419,16 @@ class Prop_accesswidth(PropertyRule):
         """
         return node.get_property('regwidth')
 
+    def validate(self, node: m_node.Node, value: Any) -> None:
+        # 10.6.1-c: The value of the accesswidth property shall not exceed the
+        # value of the regwidth property
+        if value > node.get_property('regwidth'):
+            self.env.msg.error(
+                "Register '%s' has accesswidth of %d which exceeds its regwidth of %d"
+                % (node.inst_name, value, node.get_property('regwidth')),
+                node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+            )
+
 class Prop_shared(PropertyRule):
     bindable_to = {comp.Reg}
     valid_types = (bool,)
@@ -1583,6 +1552,16 @@ class Prop_bridge(PropertyRule):
     dyn_assign_allowed = False
     mutex_group = None
 
+    def validate(self, node: m_node.Node, value: Any) -> None:
+        # 13.5: Bridge can only be applied to the root address map
+        if value:
+            # is bridge
+            if (node.parent is not None) and not isinstance(node.parent, m_node.RootNode):
+                self.env.msg.error(
+                    "The 'bridge' property can only be applied to the root address map.",
+                    node.inst.property_src_ref.get(self.get_name(), node.inst.inst_src_ref)
+                )
+
 #===============================================================================
 # User-defined property
 #===============================================================================
@@ -1797,7 +1776,7 @@ class PropertyRuleBook:
         self.rdl_prop_refs = {} # type: Dict[str, Type[rdltypes.PropertyReference]]
         for prop_ref in get_all_subclasses(rdltypes.PropertyReference):
             if prop_ref.__name__.startswith("PropRef_"):
-                prop_name = prop_ref.get_name() # type: ignore
+                prop_name = prop_ref.get_name()
                 self.rdl_prop_refs[prop_name] = prop_ref
 
     def lookup_property(self, prop_name: str) -> Optional[PropertyRule]:
@@ -1828,3 +1807,13 @@ class PropertyRuleBook:
             )
 
         self.user_properties[udp.name] = udp
+
+
+# Dictionary of mutex group names --> set of property names that belong to the mutex
+MUTEX_PROP_GROUPS = {} # type: Dict[str, Set[str]]
+for prop_cls in get_all_subclasses(PropertyRule):
+    if prop_cls.__name__.startswith("Prop_"):
+        if prop_cls.mutex_group is not None:
+            if prop_cls.mutex_group not in MUTEX_PROP_GROUPS:
+                MUTEX_PROP_GROUPS[prop_cls.mutex_group] = set()
+            MUTEX_PROP_GROUPS[prop_cls.mutex_group].add(prop_cls.get_name_cls())
