@@ -3,12 +3,11 @@ from typing import TYPE_CHECKING, List
 from .helpers import is_pow2, roundup_pow2, roundup_to
 from .. import walker
 from .. import rdltypes
-from ..node import Node, AddressableNode
+from ..node import Node, AddressableNode, SignalNode
 from ..node import AddrmapNode, RegfileNode, MemNode, RegNode, FieldNode
 
 if TYPE_CHECKING:
     from ..compiler import RDLEnvironment
-    from typing import Dict
 
 #===============================================================================
 # Validation Listeners
@@ -27,6 +26,11 @@ class ValidateListener(walker.RDLListener):
         # Same concept as the field check buffer, but is also a stack
         self.addr_check_buffer_stack = [[]] # type: List[List[AddressableNode]]
 
+        # Keep track of whether a given hierarchy has a reset signal.
+        # Signals can exist in Root, so pre-load with one stack entry
+        self.has_cpuif_reset_stack = [False] # type: List[bool]
+        self.has_field_reset_stack = [False] # type: List[bool]
+
 
     def enter_Component(self, node: Node) -> None:
         # Validate all properties that were applied to the component
@@ -39,6 +43,36 @@ class ValidateListener(walker.RDLListener):
             prop_rule = self.env.property_rules.lookup_property(prop_name)
             prop_rule.validate(node, prop_value)
 
+        if not isinstance(node, SignalNode):
+            self.has_cpuif_reset_stack.append(False)
+            self.has_field_reset_stack.append(False)
+
+
+    def enter_Signal(self, node: SignalNode) -> None:
+        if node.get_property('cpuif_reset'):
+            # 8.2.1-f: cpuif_reset property can only be set true for one
+            # instantiated signal within a lexical scope.
+            # (spec authors repeately misuse the word 'lexical', they mean hierarchical)
+            if self.has_cpuif_reset_stack[-1]:
+                self.msg.error(
+                    "Only one 'cpuif_reset' signal is allowed per hierarchy. Signal '%s' is redundant."
+                    % (node.inst_name),
+                    node.inst.inst_src_ref
+                )
+            self.has_cpuif_reset_stack[-1] = True
+
+        if node.get_property('field_reset'):
+            # 8.2.1-g: field_reset property can only be set true for one
+            # instantiated signal within a lexical scope.
+            # (spec authors repeately misuse the word 'lexical', they mean hierarchical)
+            if self.has_field_reset_stack[-1]:
+                self.msg.error(
+                    "Only one 'field_reset' signal is allowed per hierarchy. Signal '%s' is redundant."
+                    % (node.inst_name),
+                    node.inst.inst_src_ref
+                )
+            self.has_field_reset_stack[-1] = True
+
 
     def enter_AddressableComponent(self, node: AddressableNode) -> None:
         addr_check_buffer = self.addr_check_buffer_stack[-1]
@@ -61,6 +95,10 @@ class ValidateListener(walker.RDLListener):
                     ):
                         overlap_allowed = True
 
+                # Bridge addrmaps allow overlapping children
+                if isinstance(node.parent, AddrmapNode) and node.parent.get_property('bridge'):
+                    overlap_allowed = True
+
                 if not overlap_allowed:
                     self.msg.error(
                         "Instance '%s' at offset +0x%X:0x%X overlaps with '%s' at offset +0x%X:0x%X"
@@ -107,6 +145,33 @@ class ValidateListener(walker.RDLListener):
                 )
 
 
+    def enter_Addrmap(self, node: AddrmapNode) -> None:
+        if node.get_property("bridge"):
+            # This is a 'bridge addrmap'
+            # Verify that:
+            #  - Child components are only other addrmaps (signals are ok too)
+            #  - has at least 2 child addrmaps
+            n_child_addrmaps = 0
+            for child in node.children():
+                if isinstance(child, AddrmapNode):
+                    n_child_addrmaps += 1
+                elif isinstance(child, SignalNode):
+                    pass
+                else:
+                    self.msg.error(
+                        "Addrmap '%s' is a bridge which can only contain other addrmaps. Contains a child instance '%s' which is a %s"
+                        % (node.inst_name, child.inst_name, type(child.inst).__name__.lower()),
+                        child.inst.inst_src_ref
+                    )
+
+            if n_child_addrmaps < 2:
+                self.msg.error(
+                    "Addrmap '%s' is a bridge and shall contain 2 or more sub-addrmaps"
+                    % node.inst_name,
+                    node.inst.inst_src_ref
+                )
+
+
     def enter_Reg(self, node: RegNode) -> None:
         self.field_check_buffer = []
 
@@ -337,3 +402,9 @@ class ValidateListener(walker.RDLListener):
     def exit_AddressableComponent(self, node: AddressableNode) -> None:
         self.addr_check_buffer_stack.pop()
         self.addr_check_buffer_stack[-1].append(node)
+
+
+    def exit_Component(self, node: Node) -> None:
+        if not isinstance(node, SignalNode):
+            self.has_cpuif_reset_stack.pop()
+            self.has_field_reset_stack.pop()
