@@ -2,8 +2,7 @@ import re
 import itertools
 from copy import deepcopy
 from collections import deque
-from typing import TYPE_CHECKING, Optional, Iterator, Any, List, Callable, Dict
-import warnings
+from typing import TYPE_CHECKING, Optional, Iterator, Any, List, Dict, Union
 
 from . import component as comp
 from . import rdltypes
@@ -80,36 +79,6 @@ class Node:
             raise RuntimeError
 
 
-    @classmethod
-    def add_derived_property(cls, getter_function: Callable, name: Optional[str]=None) -> None:
-        """
-        Register a user-defined derived property
-
-        Parameters
-        ----------
-        getter_function : function
-            Function that fetches the result of the user-defined derived property
-        name : str
-            Derived property name
-            If unassigned, will default to the function's name
-
-        .. deprecated:: 1.15
-            Use of this function is not recommended since it encourages the bad
-            practice of modifying the SystemRDL API at runtime. Instead, it is
-            recommended that any extended property derivations be done in a
-            separate function outside of the Node interface.
-
-            In future revisions, use of this function will emit a warning, and will
-            eventually be removed entirely.
-        """
-        warnings.warn("deprecated", DeprecationWarning)
-
-        if name is None:
-            name = getter_function.__name__
-        mp = property(fget=getter_function)
-        setattr(cls, name, mp)
-
-
     def children(self, unroll: bool=False, skip_not_present: bool=True) -> Iterator['Node']:
         """
         Returns an iterator that provides nodes for all immediate children of
@@ -198,6 +167,12 @@ class Node:
         for child in self.children(skip_not_present=skip_not_present):
             if isinstance(child, SignalNode):
                 yield child
+            else:
+                # Optimization.
+                # children are sorted so signals are always first.
+                # If encountered a non-signal, no reason to continue iterating
+                break
+
 
 
     def fields(self, skip_not_present: bool=True) -> Iterator['FieldNode']:
@@ -685,6 +660,25 @@ class Node:
         """
         return self.inst.external
 
+    @property
+    def cpuif_reset(self) -> 'Optional[SignalNode]':
+        """
+        Returns a reference to the nearest cpuif_reset signal in the enclosing
+        hierarchy.
+
+        Is None if no cpuif_reset signal was found.
+
+        .. versionadded:: 1.19
+        """
+        current_node = self
+        while current_node is not None:
+            for signal in current_node.signals():
+                if signal.get_property('cpuif_reset'):
+                    return signal
+            current_node = current_node.parent
+
+        return None
+
 
     def __eq__(self, other: object) -> bool:
         """
@@ -797,11 +791,11 @@ class AddressableNode(Node):
             # Is flattened like this:
             #   idx = I0*S1*S2 + I1*S2 + I2
             idx = 0
-            for i in range(len(self.current_idx)):
+            for i, current_idx in enumerate(self.current_idx):
                 sz = 1
-                for j in range(i+1, len(self.array_dimensions)):
+                for j in range(i + 1, len(self.array_dimensions)):
                     sz *= self.array_dimensions[j]
-                idx += sz * self.current_idx[i]
+                idx += sz * current_idx
 
             offset = self.raw_address_offset + idx * self.array_stride
 
@@ -1024,6 +1018,26 @@ class FieldNode(VectorNode):
         return sw in (rdltypes.AccessType.rw, rdltypes.AccessType.rw1,
                         rdltypes.AccessType.r)
 
+    @property
+    def is_hw_writable(self) -> bool:
+        """
+        Field is writable by hardware
+        """
+        hw = self.get_property('hw')
+
+        return hw in (rdltypes.AccessType.rw, rdltypes.AccessType.rw1,
+                        rdltypes.AccessType.w, rdltypes.AccessType.w1)
+
+    @property
+    def is_hw_readable(self) -> bool:
+        """
+        Field is readable by hardware
+        """
+        hw = self.get_property('hw')
+
+        return hw in (rdltypes.AccessType.rw, rdltypes.AccessType.rw1,
+                        rdltypes.AccessType.r)
+
     @property
     def implements_storage(self) -> bool:
         """
@@ -1095,6 +1109,26 @@ class RegNode(AddressableNode):
                 return True
         return False
 
+    @property
+    def has_hw_writable(self) -> bool:
+        """
+        Register contains one or more present fields writable by hardware
+        """
+        for field in self.fields():
+            if field.is_hw_writable:
+                return True
+        return False
+
+    @property
+    def has_hw_readable(self) -> bool:
+        """
+        Register contains one or more present fields readable by hardware
+        """
+        for field in self.fields():
+            if field.is_hw_readable:
+                return True
+        return False
+
 #===============================================================================
 class RegfileNode(AddressableNode):
 
@@ -1120,16 +1154,13 @@ class MemNode(AddressableNode):
         return entry_size * num_entries
 
 #===============================================================================
-def get_group_node_size(node: AddressableNode) -> int:
+def get_group_node_size(node: Union[AddrmapNode, RegfileNode]) -> int:
     """
     Shared getter for AddrmapNode and RegfileNode's "size" property
     """
-    # After structural placement, children are sorted
-    if(not node.inst.children
-        or (not isinstance(node.inst.children[-1], comp.AddressableComponent))
-    ):
-        # No addressable child exists.
-        return 0
+    # addrmap/regfile is guaranteed to have an addressable child.
+    assert node.inst.children
+    assert isinstance(node.inst.children[-1], comp.AddressableComponent)
 
     # Current node's size is based on last child
     last_child_node = Node._factory(node.inst.children[-1], node.env, node)
